#!/usr/env/bin python
# -*- coding:utf-8 -*-

class Copyright(object):
    frame_name = "LcyFrame"
    auto_name = "Lcylln"
    auto_email = "123220663@qq.com"

    # file_pre = "# /usr/env/bin python\n# -*- coding:utf-8 -*-\n# Copyright %s %s" % (auto_name, auto_email)
    # file_pre = "#! /usr/env/bin python\n# -*- coding:utf-8 -*-"
    file_pre = ""   # p3

